package com.ekexiu.portal.cms;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.sql.DataSource;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.util.ConstData;
import org.jfw.util.json.JsonService;
import org.jfw.util.log.LogFactory;
import org.jfw.util.log.Logger;

import com.ekexiu.portal.dao.ArticleDao;
import com.ekexiu.portal.dao.PpaperDao;
import com.ekexiu.portal.dao.PpatentDao;
import com.ekexiu.portal.po.Article;
import com.ekexiu.portal.po.Ppaper;
import com.ekexiu.portal.po.Ppatent;

import freemarker.cache.TemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import freemarker.template.TemplateExceptionHandler;

@Path("/cms")
public class TemplateService {
	private static final Logger log = LogFactory.getLog(TemplateService.class);

	private final Queue<Node> queue = new ConcurrentLinkedQueue<Node>();

	@SuppressWarnings("unused")
	private File source;

	private Configuration cfg = new Configuration(Configuration.VERSION_2_3_26);

	private long rate = 10000;

	@Autowrie
	private ArticleDao articleDao;
	@Autowrie
	private PpaperDao ppaperDao;
	@Autowrie
	private PpatentDao ppatentDao;

	private File dir;

	public File getDir() {
		return dir;
	}

	public void setDir(File dir) {
		this.dir = dir;
	}

	public ArticleDao getArticleDao() {
		return articleDao;
	}

	public void setArticleDao(ArticleDao articleDao) {
		this.articleDao = articleDao;
	}

	public PpaperDao getPpaperDao() {
		return ppaperDao;
	}

	public void setPpaperDao(PpaperDao ppaperDao) {
		this.ppaperDao = ppaperDao;
	}

	public PpatentDao getPpatentDao() {
		return ppatentDao;
	}

	public void setPpatentDao(PpatentDao ppatentDao) {
		this.ppatentDao = ppatentDao;
	}

	public long getRate() {
		return rate;
	}

	public void setRate(long rate) {
		this.rate = rate;
	}

	public void setSource(File source) {
		this.source = source;
		try {
			this.cfg.setDirectoryForTemplateLoading(source);
		} catch (IOException e) {
			this.source = null;
			this.cfg.setTemplateLoader(new TemplateLoader() {
				@Override
				public Reader getReader(Object templateSource, String encoding) throws IOException {
					StringReader reader = new StringReader("TemplateService'source is invalid dir,so " + templateSource.toString() + " can't read");
					return reader;
				}

				@Override
				public long getLastModified(Object templateSource) {
					return 0;
				}

				@Override
				public Object findTemplateSource(String name) throws IOException {
					return name;
				}

				@Override
				public void closeTemplateSource(Object templateSource) throws IOException {
				}
			});
		}
		cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
	}

	public void genCnt(String templateName, File dest, Map<String, Object> root) throws IOException, TemplateException {
		if (dest.exists()) {
			if (!dest.delete()) {
				throw new IOException("delete file[" + dest.getAbsolutePath() + "] error");
			}
		}
		File parent = dest.getParentFile();
		if(!parent.exists()){
			parent.mkdirs();
		}
		Template template = this.cfg.getTemplate(templateName, "UTF-8");
		OutputStream out = new FileOutputStream(dest);
		try {
			template.process(root, new OutputStreamWriter(out, ConstData.UTF8));
			out.flush();
		} finally {
			out.close();
		}
	}

	public void push(ContentType type, Object key) {
		Node node = new Node();
		node.setType(type);
		node.setKey(key);
		this.queue.add(node);
	}

	@Get
	@Post
	@Path()
	public void push(String t, String k) {
		ContentType type = null;
		try {
			type = ContentType.valueOf(t);
		} catch (Throwable thr) {
		}
		if (type != null) {
			this.push(type, k);
		}
	}

	public void handle(DataSource ds) {
		Connection con;
		try {
			con = ds.getConnection();
		} catch (SQLException e1) {
			return;
		}
		try {
			for (;;) {
				if (!this.handle(con))
					break;
			}
		} finally {
			if (con != null) {
				try {
					con.close();
				} catch (Exception e) {
				}
			}
		}

	}

	public boolean handle(Connection con) {
		Node node = this.queue.peek();
		if (node == null)
			return false;
		try {
			ContentType type = node.getType();
			Object key = node.getKey();
			if (type != null) {
				switch (type) {
				case Article:
					this.staticArticle(con, key);
					break;
				case Paper:
					this.staticPaper(con, key);
					break;
				case Patent:
					this.staticPatent(con, key);
					break;
				default:
					break;
				}
			}
		} finally {
			if (node != null) {
				this.queue.poll();
			}
		}
		return true;
	}

	private void staticArticle(Connection con, Object key) {
		try {
			Article article = articleDao.queryOne(con, (String) key);
			if (article == null)
				return;
			String dir = article.getCreateTime().substring(0, 8);
			File file = new File(this.dir, dir + "/a/" + article.getShareId() + ".html");
			if (file.exists()) {
				file.delete();
			}
			Map<String, Object> data = new HashMap<String, Object>();
			data.put("data", article);
			data.put("jsonData", JsonService.toJson(article));
			if (article.getStatus().equals("1")) {
				this.genCnt("article.html", file, data);
			}
		} catch (Exception e) {
			log.error("static article[" + key + "]error", e);
		}
	}

	private void staticPaper(Connection con, Object key) {
		try {
			Ppaper page = this.ppaperDao.query(con, (String) key);
			if (page == null)
				return;
			String dir = page.getCreateTime().substring(0, 8);
			File file = new File(this.dir, dir + "/p/" + page.getShareId() + ".html");
			if (file.exists()) {
				file.delete();
			}
			Map<String, Object> data = new HashMap<String, Object>();
			data.put("data", page);
			data.put("jsonData", JsonService.toJson(page));
			this.genCnt("paper.html", file, data);
		} catch (Exception e) {
			log.error("static paper[" + key + "]error", e);
		}
	}

	private void staticPatent(Connection con, Object key) {
		try {
			Ppatent page = this.ppatentDao.query(con, (String) key);
			if (page == null)
				return;
			String dir = page.getCreateTime().substring(0, 8);
			File file = new File(this.dir, dir + "/p/" + page.getShareId() + ".html");
			if (file.exists()) {
				file.delete();
			}
			Map<String, Object> data = new HashMap<String, Object>();
			data.put("data", page);
			data.put("jsonData", JsonService.toJson(page));
			this.genCnt("patent.html", file, data);
		} catch (Exception e) {
			log.error("static patent[" + key + "]error", e);
		}
	}

	public static void main(String[] args) throws Exception {
		System.out.println(ContentType.valueOf("Article"));

	}

}
